<?php

declare(strict_types=1);

namespace Drupal\track_usage\Plugin\TrackUsage\Track;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\track_usage\Attribute\Track;
use Drupal\track_usage\EntityGuesser;
use Drupal\track_usage\EntityGuesserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Tracks usage of entities referenced from regular HTML links.
 */
#[Track(
  id: 'html_link',
  label: new TranslatableMarkup('HTML links'),
  fieldTypes: ['text', 'text_long', 'text_with_summary'],
)]
class HtmlLink extends TextFieldBase {

  public function __construct(
    array $configuration,
    string $plugin_id,
    mixed $plugin_definition,
    EntityRepositoryInterface $entityRepository,
    EntityTypeManagerInterface $entityTypeManager,
    protected readonly EntityGuesserInterface $entityGuesser,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entityRepository, $entityTypeManager);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity.repository'),
      $container->get('entity_type.manager'),
      $container->get(EntityGuesser::class),
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function xpath(): string {
    return "//a[@href != '']";
  }

  /**
   * {@inheritdoc}
   */
  protected function getEntity(\DOMNode $node): ?EntityInterface {
    return $this->entityGuesser->guessFromUrl($node->getAttribute('href'));
  }

}
