<?php

declare(strict_types=1);

namespace Drupal\track_usage;

use Drupal\Core\Entity\EntityInterface;
use Drupal\track_usage\Entity\TrackConfigInterface;
use Drupal\track_usage\Model\Operation;

/**
 * Interface for `track_usage.recorder` service.
 */
interface RecorderInterface extends ReaderInterface {

  /**
   * Table for usage storage.
   */
  public const TABLE = 'track_usage';

  /**
   * Records target entity usages of an entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity for which to register target usages.
   * @param \Drupal\track_usage\Entity\TrackConfigInterface $config
   *   The track configuration.
   */
  public function record(EntityInterface $entity, TrackConfigInterface $config): void;

  /**
   * Records target entity usages of an entity given entity data.
   *
   * This is an alternative to self::record() which doesn't expect an entity
   * object but the entity type ID, the entity ID and the entity revision.
   *
   * @param string $entityTypeId
   *   The entity type.
   * @param int|string $entityId
   *   The entity ID.
   * @param int|string $revisionId
   *   The entity revision ID.
   * @param string $configId
   *   The configuration ID.
   */
  public function recordRaw(string $entityTypeId, int|string $entityId, int|string $revisionId, string $configId): void;

  /**
   * Cleans up the usage records.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity for which to register target usages.
   * @param \Drupal\track_usage\Model\Operation $operation
   *   The operation.
   */
  public function cleanup(EntityInterface $entity, Operation $operation): void;

}
